/*******************************************************************************
 * Copyright (c) 2005, 2007 Spring IDE Developers
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Spring IDE Developers - initial API and implementation
 *******************************************************************************/
package org.springframework.ide.eclipse.webflow.ui.editor.hyperlink.webflow;

import java.util.LinkedHashSet;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.hyperlink.IHyperlink;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;
import org.springframework.ide.eclipse.beans.core.internal.model.BeansModelUtils;
import org.springframework.ide.eclipse.beans.core.model.IBean;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.AbstractHyperlinkDetector;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.ExternalBeanHyperlink;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.JavaElementHyperlink;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.NodeElementHyperlink;
import org.springframework.ide.eclipse.beans.ui.editor.util.BeansEditorUtils;
import org.springframework.ide.eclipse.core.java.Introspector;
import org.springframework.ide.eclipse.core.java.JdtUtils;
import org.springframework.ide.eclipse.ui.SpringUIUtils;
import org.springframework.ide.eclipse.webflow.core.Activator;
import org.springframework.ide.eclipse.webflow.core.internal.model.WebflowModelUtils;
import org.springframework.ide.eclipse.webflow.core.model.IWebflowConfig;
import org.springframework.ide.eclipse.webflow.ui.editor.WebflowNamespaceUtils;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Detects hyperlinks in XML tags. Includes detection of bean classes and bean properties in
 * attribute values. Resolves bean references (including references to parent beans or factory
 * beans).
 * @author Christian Dupuis
 */
public class WebflowHyperlinkDetector extends AbstractHyperlinkDetector implements
		IHyperlinkDetector {

	private static final Set<String> VALID_ATTRIBUTES;

	static {
		VALID_ATTRIBUTES = new LinkedHashSet<String>();
		// web flow 1.x
		VALID_ATTRIBUTES.add("bean");
		VALID_ATTRIBUTES.add("method");
		VALID_ATTRIBUTES.add("to");
		VALID_ATTRIBUTES.add("on-exception");
		VALID_ATTRIBUTES.add("from");
		VALID_ATTRIBUTES.add("to");
		VALID_ATTRIBUTES.add("idref");
		VALID_ATTRIBUTES.add("class");
		VALID_ATTRIBUTES.add("then");
		VALID_ATTRIBUTES.add("else");
		VALID_ATTRIBUTES.add("type");
		VALID_ATTRIBUTES.add("name");
		VALID_ATTRIBUTES.add("flow");

		// web flow 2.x
		VALID_ATTRIBUTES.add("type");
		VALID_ATTRIBUTES.add("class");
		VALID_ATTRIBUTES.add("result-type");
		VALID_ATTRIBUTES.add("bean");
		VALID_ATTRIBUTES.add("subflow-attribute-mapper");

		VALID_ATTRIBUTES.add("to");
		VALID_ATTRIBUTES.add("start-state");
		VALID_ATTRIBUTES.add("then");
		VALID_ATTRIBUTES.add("else");

		VALID_ATTRIBUTES.add("subflow");
		VALID_ATTRIBUTES.add("parent");
	}

	/**
	 * Returns <code>true</code> if given attribute is openable.
	 * @param attr
	 * @return
	 */
	public boolean isLinkableAttr(Attr attr) {
		return VALID_ATTRIBUTES.contains(attr.getLocalName());
	}

	public IHyperlink createHyperlink(final String name, final String target, Node node,
			Node parentNode, IDocument document, ITextViewer textViewer,
			final IRegion hyperlinkRegion, IRegion cursor) {
		if (name == null) {
			return null;
		}
		if ("bean".equals(name) || "name".equals(name) || "subflow-attribute-mapper".equals(name)) {
			IFile file = BeansEditorUtils.getFile(document);
			IWebflowConfig config = Activator.getModel().getProject(file.getProject()).getConfig(
					file);
			if (config != null) {
				Set<IBean> beans = WebflowModelUtils.getBeans(config);
				for (IBean bean : beans) {
					if (bean.getElementName().equals(target)) {
						return new ExternalBeanHyperlink(bean, hyperlinkRegion);
					}
				}
			}
		}
		else if (("to".equals(name) && "transition".equals(node.getLocalName()))
				|| "then".equals(name) || "else".equals(name) || "idref".equals(name)
				|| "start-state".equals(name)) {
			Node flowNode = WebflowNamespaceUtils.locateFlowRootNode(node);
			NodeList nodes = flowNode.getChildNodes();
			if (nodes.getLength() > 0) {
				for (int i = 0; i < nodes.getLength(); i++) {
					String id = BeansEditorUtils.getAttribute(nodes.item(i), "id");
					if (target.equals(id)) {
						IRegion region = getHyperlinkRegion(nodes.item(i));
						return new NodeElementHyperlink(hyperlinkRegion, region, textViewer);
					}
				}
			}
		}
		else if (("to".equals(name) && !"transition".equals(node.getLocalName()))
				|| "on-exception".equals(name) || "type".equals(name) || "type".equals(name)
				|| "class".equals(name) || "result-type".equals(name)) {
			IFile file = BeansEditorUtils.getFile(document);
			IType type = JdtUtils.getJavaType(file.getProject(), target);
			if (type != null) {
				return new JavaElementHyperlink(hyperlinkRegion, type);
			}
		}
		else if ("method".equals(name) && BeansEditorUtils.hasAttribute(node, "bean")) {
			String bean = BeansEditorUtils.getAttribute(node, "bean");
			IFile file = BeansEditorUtils.getFile(document);
			String className = null;
			IWebflowConfig config = Activator.getModel().getProject(file.getProject()).getConfig(
					file);
			if (config != null) {
				Set<IBean> beans = WebflowModelUtils.getBeans(config);
				for (IBean modelBean : beans) {
					if (modelBean.getElementName().equals(bean)) {
						className = BeansModelUtils.getBeanClass(modelBean, null);
					}
				}
				IType type = JdtUtils.getJavaType(file.getProject(), className);
				if (type != null) {
					try {
						Set<IMethod> methods = Introspector.getAllMethods(type);
						if (methods != null) {
							for (IMethod method : methods) {
								if (method.getElementName().equals(target)) {
									return new JavaElementHyperlink(hyperlinkRegion, method);
								}
							}
						}
					}
					catch (JavaModelException e) {
					}
				}
			}
		}
		else if ("flow".equals(name) || "subflow".equals(name) || "parent".equals(name)) {
			IFile file = BeansEditorUtils.getFile(document);
			final IWebflowConfig config = Activator.getModel().getProject(file.getProject())
					.getConfig(target);
			if (config != null) {
				return new IHyperlink() {

					public IRegion getHyperlinkRegion() {
						return hyperlinkRegion;
					}

					public String getHyperlinkText() {
						return target;
					}

					public String getTypeLabel() {
						return name;
					}

					public void open() {
						SpringUIUtils.openInEditor(config.getResource(), -1);
					}
				};
			}
		}
		return null;
	}
}
